#include "node/state_machines/base.h"
#include "node/state_machines/interface.h"

#pragma once

namespace schpin
{

class boot_state_machine : state_machine_i
{
        bool                    started_ = false;
        std::vector< servo_id > to_finish_{};
        std::vector< servo_id > to_activate_;
        bool                    finished_ = false;

      public:
        boot_state_machine( std::vector< servo_id > to_activate )
          : to_activate_( std::move( to_activate ) )
        {
        }

        opt_error< state_machine_error > on_cmd( fw_node_group_variant v,
                                                 const node_context&,
                                                 sm_feedback_f )
        {
                return v << tuple_match(
                           [&]( cmd_tag< ACTIVATE >,
                                servo_id       id,
                                activated_enum a ) -> opt_error< state_machine_error > {
                                   if ( a == WAS_ACTIVATING ) {
                                           SCHPIN_INFO_LOG( "boot", "Allready activating: " << id );
                                           return {};
                                   }
                                   if ( a == ACTIVATING ) {
                                           SCHPIN_INFO_LOG( "boot",
                                                            "Started activation of: " << id );
                                           return {};
                                   }
                                   auto iter = em::find( to_finish_, id );
                                   if ( iter == to_finish_.end() ) {
                                           return { E( state_machine_error )  //
                                                    << "Servo which should not activate ,was "
                                                       "activating: "
                                                    << id };
                                   }
                                   if ( a == MISSING ) {
                                           SCHPIN_INFO_LOG( "boot", "Servo " << id << " missing." );
                                   } else if ( a == ACTIVATED ) {
                                           SCHPIN_INFO_LOG( "boot", "Activated: " << id );
                                   } else {
                                           SCHPIN_INFO_LOG( "boot", "Allready activated: " << id );
                                   }
                                   to_finish_.erase( iter );
                                   if ( to_finish_.empty() ) {
                                           finished_ = true;
                                   }
                                   return {};
                           },
                           [&]( auto... ) -> opt_error< state_machine_error > {
                                   return {};
                           } );
        }

        opt_error< state_machine_error > tick( const node_context&, sm_feedback_f send_cb )
        {
                if ( started_ ) {
                        SCHPIN_INFO_LOG_THROTTLE(
                            60,
                            "boot_state_machine",
                            "Waiting for boot of: " << em::view{ to_finish_ } );
                        return {};
                }
                SCHPIN_INFO_LOG( "boot",
                                 "boot executed, activating: " << em::view{ to_activate_ } );
                for ( servo_id id : to_activate_ ) {
                        send_cb(
                            node_fw_group::create< ACTIVATE >( *lewan_servo_id::make( id ) ) );
                }
                to_finish_ = to_activate_;

                started_ = true;
                return {};
        }

        bool finished() final
        {
                return finished_;
        }
};

}  // namespace schpin
